/*
This file is part of the Juju GUI, which lets users view and manage Juju
environments within a graphical interface (https://launchpad.net/juju-gui).
Copyright (C) 2015 Canonical Ltd.

This program is free software: you can redistribute it and/or modify it under
the terms of the GNU Affero General Public License version 3, as published by
the Free Software Foundation.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero
General Public License for more details.

You should have received a copy of the GNU Affero General Public License along
with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

'use strict';

YUI.add('deployment-summary', function () {

  juju.components.DeploymentSummary = React.createClass({

    propTypes: {
      jem: React.PropTypes.object.isRequired,
      env: React.PropTypes.object.isRequired,
      appSet: React.PropTypes.func.isRequired,
      createSocketURL: React.PropTypes.func.isRequired,
      deploymentStorage: React.PropTypes.object.isRequired,
      users: React.PropTypes.object.isRequired,
      autoPlaceUnits: React.PropTypes.func.isRequired,
      changeDescriptions: React.PropTypes.array.isRequired,
      changeState: React.PropTypes.func.isRequired,
      ecsClear: React.PropTypes.func.isRequired,
      ecsCommit: React.PropTypes.func.isRequired,
      getUnplacedUnitCount: React.PropTypes.func.isRequired,
      modelCommitted: React.PropTypes.bool.isRequired,
      numberOfChanges: React.PropTypes.number.isRequired
    },

    /**
      Close the summary.
       @method _close
    */
    _close: function _close() {
      this.props.changeState({
        sectionC: {
          component: null,
          metadata: {}
        }
      });
    },

    /**
      Handles calling to clear the ecs and then closing the deployment
      summary.
       @method _handleClear
    */
    _handleClear: function _handleClear() {
      this.props.ecsClear();
      this._close();
    },

    /**
      Navigate to the choose cloud step.
       @method _handleChangeCloud
    */
    _handleChangeCloud: function _handleChangeCloud() {
      this.props.changeState({
        sectionC: {
          component: 'deploy',
          metadata: {
            activeComponent: 'choose-cloud'
          }
        }
      });
    },

    /**
      Handle committing when the deploy button in the summary is clicked.
       @method _handleDeploy
    */
    _handleDeploy: function _handleDeploy() {
      var _this = this;

      this.props.autoPlaceUnits();
      // The env is already bound to ecsCommit in app.js.
      // Generates an alphanumeric string
      var randomString = function randomString() {
        return Math.random().toString(36).slice(2);
      };
      var password = randomString() + randomString();
      this.props.jem.newEnvironment(this.props.users.jem.user,
      // XXX Hardcoding the model name because we don't yet have a field
      // for it to be inputted.
      'my-test-model', this.props.deploymentStorage.templateName,
      // XXX Hardcoding the controller for now but it will be provided on load
      'yellow/aws-eu-central', password, function (error, data) {
        if (error) throw error;
        var pathParts = data['host-ports'][0].split(':');
        // Set the credentials in the env so that the GUI
        // is able to connect to the new model.
        _this.props.env.setCredentials({
          user: 'user-' + data.user,
          password: data.password
        });
        var socketURL = _this.props.createSocketURL(pathParts[0], // server
        pathParts[1], // port
        data.uuid);
        // Set the socket url in both the app and the env so we don't end
        // up with any confusion later on about which is which.
        _this.props.appSet('socket_url', socketURL);
        _this.props.env.set('socket_url', socketURL);
        _this.props.env.connect();
        // After the model connects it will emit a login event, listen
        // for that event so that we know when to commit the changeset.
        _this.props.env.on('login', function (data) {
          _this.props.ecsCommit();
          _this.setState({ hasCommits: true }, function () {
            _this._close();
          });
        });
      });
    },

    /**
      Handle navigating to the machine view.
       @method _handleViewMachinesClick
    */
    _handleViewMachinesClick: function _handleViewMachinesClick() {
      this.props.changeState({
        sectionB: {
          component: 'machine',
          metadata: {}
        },
        sectionC: {
          component: null,
          metadata: {}
        }
      });
    },

    /**
      Generate the list of change items.
       @method _generateChangeItems
      @returns {Array} The collection of changes.
    */
    _generateChangeItems: function _generateChangeItems() {
      var changeList = this.props.changeDescriptions;
      var changes = [];
      changeList.forEach(function (change, i) {
        changes.push(React.createElement(juju.components.DeploymentSummaryChangeItem, {
          key: i,
          change: change }));
      }, this);
      return changes;
    },

    /**
      Generate a message if there are unplaced units.
       @method _generatePlacement
      @returns {Object} The placement markup.
    */
    _generatePlacement: function _generatePlacement() {
      var unplacedCount = this.props.getUnplacedUnitCount();
      if (unplacedCount === 0) {
        return;
      }
      var plural = unplacedCount === 1 ? '' : 's';
      return React.createElement(
        'div',
        { className: 'deployment-summary__placement twelve-col' },
        React.createElement(
          'span',
          null,
          'You have ',
          unplacedCount.toString(),
          ' unplaced unit',
          plural,
          ' which will be automatically placed.'
        ),
        React.createElement(
          'span',
          { className: 'link', tabIndex: '0', role: 'button',
            onClick: this._handleViewMachinesClick },
          'View machines'
        )
      );
    },

    /**
      Generate a clear changes control when showing the commit flow.
       @method _generateClearChanges
      @returns {Object} The clear changes control.
    */
    _generateClearChanges: function _generateClearChanges() {
      var modelCommitted = this.props.modelCommitted;
      // Don't show the clear changes control if we're deploying the model for
      // the first time.
      if (!modelCommitted) {
        return;
      }
      return React.createElement(
        'span',
        { className: 'link deployment-panel__section-title-link',
          onClick: this._handleClear,
          role: 'button',
          tabIndex: '0' },
        'Clear all changes ›'
      );
    },

    render: function render() {
      var listHeaderClassName = 'deployment-summary-change-item ' + 'deployment-summary__list-header';
      var buttons = [];
      var modelCommitted = this.props.modelCommitted;
      if (!modelCommitted) {
        buttons.push({
          action: this._handleChangeCloud,
          title: 'Change cloud',
          type: 'inline-neutral'
        });
      }
      buttons.push({
        title: modelCommitted ? 'Commit' : 'Deploy',
        action: this._handleDeploy,
        disabled: this.props.numberOfChanges === 0,
        type: 'inline-positive'
      });
      return React.createElement(
        'div',
        { className: 'deployment-panel__child' },
        React.createElement(
          juju.components.DeploymentPanelContent,
          {
            title: 'Deployment summary' },
          React.createElement(
            'div',
            { className: 'deployment-panel__notice twelve-col' },
            React.createElement(juju.components.SvgIcon, {
              name: 'general-action-blue',
              size: '16' }),
            'This deployment is free, you can deploy xxxxxxxxx more'
          ),
          React.createElement(
            'form',
            { className: 'six-col' },
            React.createElement(
              'p',
              null,
              'Name your model'
            ),
            React.createElement(
              'label',
              { className: 'deployment-panel__label',
                htmlFor: 'model-name' },
              'Model name'
            ),
            React.createElement('input', { className: 'deployment-panel__input',
              id: 'model-name',
              placeholder: 'test_model_01',
              type: 'text' })
          ),
          React.createElement(
            'div',
            { className: 'six-col last-col' },
            React.createElement(
              'p',
              null,
              'Deploying to:'
            ),
            React.createElement(
              'div',
              { className: 'deployment-panel__box' },
              '[selected credential]'
            )
          ),
          this._generatePlacement(),
          React.createElement(
            'h3',
            { className: 'deployment-panel__section-title' },
            'Change log (',
            this.props.numberOfChanges,
            ')',
            this._generateClearChanges()
          ),
          React.createElement(
            'ul',
            { className: 'deployment-summary__list' },
            React.createElement(
              'li',
              { className: listHeaderClassName },
              React.createElement(
                'span',
                { className: 'deployment-summary-change-item__change' },
                'Change'
              ),
              React.createElement(
                'span',
                { className: 'deployment-summary-change-item__time' },
                'Time'
              )
            ),
            this._generateChangeItems()
          )
        ),
        React.createElement(juju.components.DeploymentPanelFooter, {
          buttons: buttons })
      );
    }
  });
}, '0.1.0', { requires: ['deployment-panel-content', 'deployment-panel-footer', 'deployment-summary-change-item'] });